﻿Function Set-ConsoleWidth
{
  param
  (
    [Int]
    # gewünschte Breite der Konsole, optional
    # Vorgabe ist die sichtbare Breite:
    $Width = $host.ui.RawUI.WindowSize.Width,

    [Switch]
    # automatisch Konsole so breit wie möglich machen
    $Maximum
  )

  # maximal mögliche Breite der Konsole:
  $maximumWidth = $host.ui.RawUI.MaxPhysicalWindowSize.Width

  # aktuelle Breite der Konsole:
  $currentWindowWidth = $host.ui.RawUI.WindowSize.Width

  # aktuelle Breite des Puffers:
  $currentBufferWidth = $host.ui.RawUI.BufferSize.Width

  # maximale Breite gewünscht?
  if ($Maximum)
  {
    $Width = $maximumWidth
  }
  # Wunschbreite angegeben?
  elseif ($Width)
  {
    # falls größer als maximal erlaubt, begrenzen:
    if ($Width -gt $maximumWidth)
    {
      $Width = $maximumWidth
    }
  }

  # die Logik zum Anpassen des Buffers und des Fensters
  # sind interne Funktionen, weil diese je nach 
  # aktuellen Werten in anderer Reihenfolge
  # aufgerufen werden müssen:
  function AdjustBuffer
  {
      # Konsolenpuffer auf neue Breite setzen:
      $buffer = $host.UI.RawUI.BufferSize
      $buffer.Width = $Width
      $host.UI.RawUI.BufferSize = $buffer
  }

  function AdjustWindow
  {
      # Sichtbaren Bereich anpassen
      $buffer = $host.UI.RawUI.WindowSize
      $buffer.Width = $Width
      $host.UI.RawUI.WindowSize = $buffer
  }

  # falls das Fenster aktuell GRÖSSER ist als die 
  # neue gewünschte Breite, dann muss das Fenster
  # ZUERST verkleinert werden, bevor der Puffer
  # verkleinert werden darf:
  if ($currentWindowWidth -gt $Width)
  {
    AdjustWindow
    AdjustBuffer
  }
  # andernfalls umgekehrt:
  else
  {
    AdjustBuffer
    AdjustWindow
  }
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
