﻿Function Get-PopularName
{
  param
  (
    [ValidateSet('1880','1890','1900','1910','1920','1930','1940','1950','1960','1970','1980','1990','2000','2010')]
    $Decade = '1950'
  )

  # Muster der Information im Quelltext der Webseite
  # runde Klammern markieren Informationen, die separat extrahiert werden sollen (außer solche, die mit "?:" markiert sind)
  # (.*?) steht für "beliebiger Text"
  # (?si) steht für "single line mode" und "case-insensitive"
  $pattern = '(?si)<td>(\d{1,3})</td>\s*?<td align="center">(.*?)</td>\s*?<td>((?:\d{0,3}\,)*\d{1,3})</td>\s*?<td align="center">(.*?)</td>\s*?<td>((?:\d{0,3}\,)*\d{1,3})</td></tr>'

  # WebClient besorgen und universell konfigurieren:
  $webclient = New-Object System.Net.WebClient
  try
  {
    $proxy = [System.Net.WebRequest]::GetSystemWebProxy()
    $proxy.Credentials = [System.Net.CredentialCache]::DefaultNetworkCredentials
    $webclient.proxy = $proxy
  } catch {}
  $webclient.UseDefaultCredentials = $true

  # Quelltext der Webseite herunterladen:
  $html = $webclient.DownloadString("http://www.ssa.gov/OACT/babynames/decades/names$($decade)s.html")

  # RegEx-Engine durch Typumwandlung erstellen:
  $regex = [regex]$pattern

  # Treffer im Text suchen:
  $Matches = $regex.Matches($html)

  # Ergebnisse auswerten und in Objekte verpacken:
  $matches | ForEach-Object {
    $match = $_

    $rv = New-Object PSObject | Select-Object -Property Name, Rank, Number, Gender
    $rv.Rank = $match.Groups[1].Value
    $rv.Gender = 'm'
    $rv.Name = $match.Groups[2].Value
    $rv.Number = [Int]$match.Groups[3].Value
    $rv

    $rv = New-Object PSObject | Select-Object -Property Name, Rank, Number, Gender
    $rv.Rank = $match.Groups[1].Value
    $rv.Gender = 'f'
    $rv.Name = $match.Groups[4].Value
    $rv.Number = [Int]$match.Groups[5].Value
    $rv
  } | Sort-Object Name, Rank
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
