﻿Function Get-UserAccount
{
  [CmdletBinding(DefaultParameterSetName='localAccount')]

  param
  (
    [Parameter(Position=0)]
    $UserName=$env:UserName,

    [Parameter(ParameterSetName='localAccount')]
    $ComputerName=$env:Computername,

    [Parameter(ParameterSetName='domainAccount')]
    $DomainName=$env:UserDomain
  )

  # handelt es sich um ein Domänenkonto oder lokales Konto?
  $type = $PSCmdlet.ParameterSetName

  # wirklich? Wenn der Computername und der Domänenname identisch
  # sind, ist es ein lokales Konto:

  if ($DomainName -eq $ComputerName)
  {
    $type = 'localAccount'
  }

  if ($type -eq 'localAccount')
  {
    # lokales Konto direkt vom Computer lesen:
    $computer = [ADSI]"WinNT://$Computername,computer"
    $computer.Children.Find($UserName, 'user')
  }
  else
  {
    # Domänenkonto aus der Domäne lesen:
    $domain = [ADSI]"WinNT://$DomainName,domain"
    $domain.Children.Find($UserName, 'user')
  }
}

Function Test-UserPassword
{
  param
  (
    [Parameter(Mandatory=$true)]
    $Password,

    [Parameter(ValueFromPipeline=$true)]
    $InputObject
  )

  Process
  {
    try
    {
      # versuchsweise vermutetes Kennwort erneut setzen:
      $InputObject.ChangePassword($Password, $Password)
      # wenn erfolgreich, dann stimmt es:
      $true
    }
    catch
    {
        # bei Fehlermeldungen: die folgenden beiden besagen,
        # dass das Kennwort nicht geändert werden konnte
        # (zum Beispiel wegen Komplexitätskriterien oder weil
        # das Kennwort in der Kennworthistory gesperrt ist)
        # das Kennwort stimmt dann aber trotz des Fehlers:
        $errorcode = $_.Exception.InnerException.ErrorCode
        (($errorcode -eq 0x8007202f) -or ($errorcode -eq 0x800708c5))
    }
  }
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
