﻿Function ConvertTo-DateTimeFormat
{
  [CmdletBinding(DefaultParameterSetName='Specific')]
  param
  (
    [Parameter(Mandatory=$true,ValueFromPipeline=$true)]
    [DateTime]
    $DateTime,

    [ValidateSet('UniversalSortableDateTime', 'SortableDateTime', 'FullDateTime', 'RFC1123', 'LongDate', 'LongTime', 'ShortDate','ShortTime','YearMonth', 'MonthDay')]
    $Format = 'SortableDateTime',

    [String]
    [Parameter(ParameterSetName='Specific')]
    $Culture = [System.Globalization.CultureInfo]::CurrentUICulture.Name,

    [Switch]
    [Parameter(ParameterSetName='Invariant')]
    $Neutral
  )

  Begin
  {
    # die Eigenschaften, die die Muster enthalten, heißen so wie in $Format angegeben, allerdings
    # um das Wort "Pattern" ergänzt:
    $FormatName = $Format + 'Pattern'

    # Das Muster des gewählten Formats auslesen:
    if ($Neutral) { $Culture = '' }
   
    # wurde kein Kultur-Kurzname angegeben, sondern ein Land?
    try
    {
        $CultureInfo = [System.Globalization.CultureInfo]::GetCultureInfo($Culture)
    }
    catch
    {
        # ja, liess sich nicht umwandeln, also suchen:
        $CultureInfo = [System.Globalization.CultureInfo]::GetCultures('InstalledWin32Cultures') |
          Where-Object { $_.DisplayName -like "*$Culture*" -or $_.NativeName -like "*$Culture*" } |
          Select-Object -First 1
    }
    
    $Pattern = $CultureInfo.DateTimeFormat.$FormatName

    # wenn der Anwender -Verbose angibt, werden das verwendete Muster und die Kultur angegeben:
    Write-Verbose "Das verwendete Muster lautet: '$Pattern'"
    Write-Verbose "Die verwendete Kultur lautet: '$CultureInfo'"
  }

  Process
  {
    # Datum/Zeit in $DateTime mit toString() in gewünschter Form und Kultur ausgeben:
    $DateTime.ToString($Pattern, $CultureInfo)
  }
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
