﻿Function Find-TypeByCommandName
{
  param
  (
    [Parameter(Mandatory=$true)]
    $Keyword
  )

  # diese Methoden sind zu allgemein, ausschließen:
  $excludeAll = 'Invoke|InitializeLifetimeService|GetType|GetHashCode|Equals|Dispose'

  # diese Namensendung wird für asynchrone Aufrufe benötigt, ausschließen:
  $excludeEnding = 'Async'

  # diese Präfixe sind Methoden, die Eigenschaften und Operatoren abbilden. Ausschließen:
  $excludeStarting = 'get_|set_|op_|add_|remove_'

  [AppDomain]::CurrentDomain.GetAssemblies() |
  ForEach-Object { try { $_.GetExportedTypes() } catch {} } |
  Where-Object { $_.isPublic} |
  Where-Object { $_.isClass } |
  Where-Object { $_.Name -notmatch '(Attribute|Handler|Args|Exception|Collection|Expression)$' }|
  # nur Methoden, die dem Schlüsselwort entsprechen, und Doppelgänger ausschließen:
  Foreach-Object { $_.GetMethods() | Where-Object { $_.Name -like $Keyword } | Sort-Object -Property Name -Unique } |
  # die allgemeinen Methoden nach den Ausschlusslisten entfernen:
  Where-Object { $_.Name -notmatch "^($excludeStarting)" } |
  Where-Object { $_.Name -notmatch "($excludeEnding)$" } |
  Where-Object { $_.Name -notmatch "$excludeAll" } |
  Select-Object -Property Name, DeclaringType
}

Find-TypeByCommandName '*Network*' | Out-GridView


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
