﻿Function Get-PowerShellExample
{
    param
    (
        $RegExFilter = '.'
    )
  Add-Type -AssemblyName System.Web
  $ProgressPreference = 'SilentlyContinue'

  [AppDomain]::CurrentDomain.GetAssemblies() | 
  ForEach-Object { $_.GetExportedTypes() }  |
  Where-Object { $_.isPublic} |
  Where-Object { $_.isClass } |
  Where-Object { $_.Name -notmatch '(Attribute|Handler|Args|Exception|Collection|Expression|Parser|Statement)$' } |
  Where-Object { $_.Name -notlike '*``*' } |
  Where-Object { $_.Fullname -match $RegExFilter } |
  Select-Object -Property Name, FullName |
  ForEach-Object {
    $TypeName = $_.Name
    $suche = '"New-Object {0}" PowerShell' -f $_.FullName
    $url = 'http://www.google.de/search?hl=de&as_q={0}' -f [System.Web.HttpUtility]::UrlEncode($suche)
    $result = Invoke-WebRequest -Uri $url
    $anzahl = if ( $result.Content -match '((?:\d{1,3}\.)*\d{1,3})\s*Ergebnisse' ) { [Int]($matches[1] -replace '[.,]') } else { 
    -1
    
     }
    Write-Host ('Zu "{0}" wurden {1:n} Treffer gefunden.' -f $TypeName, $anzahl) -ForegroundColor DarkYellow
    $result.Links |
    Where-Object { $_.href.StartsWith('/url?') } |
    ForEach-Object {
      $rv = 1 | Select-Object -Property TypeName, Thema, URL, Anzahl, Object, Type
      $rv.TypeName = $TypeName
      $rv.Anzahl = $anzahl
      $rv.URL =

      if ($_.href -match 'q=(.*?);')
      {
        $matches[1]
      }
      else
      {
        $_.href
      }

      $rv.Thema = $_.outerText
      $rv.Object = $true
      $rv.Type = $false
      $rv
    }
  }
}

$OutputPath = Split-Path $profile
$OutputFile = Join-Path -Path $OutputPath -ChildPath 'weblinks.csv'

if ( (Test-Path $OutputPath) -eq $false)
{
    $null = New-Item -Path $OutputPath -ItemType Directory -Force
}

Get-PowerShellExample | 
  Out-GridView -PassThru |
  Export-CSV -Path $OutputFile -Encoding UTF8 -NoTypeInformation -UseCulture



################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
