﻿Function Convert-PictureFile
{
  param
  (
    [Parameter(Mandatory=$true,
      ValueFromPipelineByPropertyName=$true,
      ValueFromPipeline=$true,
      Position=0)]
    [Alias('FullName')]
    [String]
    $Path,

    $TargetFolder = "$env:temp\TargetFolder", 

    [ValidateSet('BMP', 'JPEG', 'GIF', 'TIFF', 'PNG')]
    $TargetFormat = 'PNG'
  )

  Begin
  {
    Add-Type -AssemblyName 'System.Drawing'
    $ImageCodec = [System.Drawing.Imaging.ImageCodecInfo]::GetImageEncoders() |
        Where-Object { $_.FormatDescription -eq $TargetFormat }

    $TargetExtension = $ImageCodec.FilenameExtension.Replace('*','').Split(';')
    if ( (Test-Path -Path $TargetFolder) -eq $false)
    { $null = New-Item -Path $TargetFolder -ItemType Directory }
  }

  Process
  {
    if ($TargetExtension -notcontains [System.IO.Path]::GetExtension($Path))
    {
      $quellname = [System.IO.Path]::GetFileName($Path)
      $zielname = [System.IO.Path]::ChangeExtension($quellname, $TargetExtension[0])
      Write-Host "Konvertiere '$quellname'..." -NoNewline
      $NewPath = Join-Path -Path $TargetFolder -Child $zielname
      $Image = [system.drawing.image]::FromFile($Path)
      $Image.Save($NewPath, $ImageCodec.FormatId)
      $Image.Dispose()
      Write-Host "nach '$zielname': OK."
    }
  }
}

Get-ChildItem -Path $env:windir -Filter *.jpg -Recurse -ErrorAction SilentlyContinue |
  Convert-PictureFile -TargetFormat PNG -TargetFolder $env:temp\Pictures

Invoke-Item -Path $env:temp\Pictures



################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
