# Ordner bestimmen, in dem Skript sich befindet:
# (in Modulen ist das der Inhalt von $PSScriptRoot)
$SelfPath = Split-Path $MyInvocation.MyCommand.Definition

# DLL nachladen:
Add-Type -Path "$SelfPath\Ionic.Zip.dll"

Function ConvertTo-ZIP
{
  [CmdletBinding()]

  param
  (
    [Parameter(Mandatory=$true)]
    # Pfadname zur ZIP-Datei
    $ZipPath,

    [Parameter(ValueFromPipeline=$true,Mandatory=$true)]
    # Pfadname der zu verpackenden Datei
    $Path,

    [Switch]
    # selbstextrahierendes Archiv erzeugen
    $SelfExtract
  )

  Begin
  {
    # Neues Objekt vom geladenen Typ "Ionic.Zip.ZipFile" herstellen:
    $zipfile = New-Object Ionic.Zip.ZipFile
  }

  Process
  {
    $Path | ForEach-Object {
      $FileObject = $_

      # wurde ein Textpfad angegeben?
      if ($FileObject -is [String])
      {
        # ja, dann Dateiobjekt beschaffen, wenn es existiert:
        if ((Test-Path -Path $FileObject))
        {
          $FileObject = Get-Item $FileObject
        }
      }

      # Ordner oder Datei?
      try {
        if ($FileObject -is [System.IO.DirectoryInfo])
        {
          # Ordner, also im ZIP-File einen Ordner anlegen:
          $e = $zipfile.AddDirectory($FileObject.FullName, '')
        }
        elseif ($FileObject -is [System.IO.FileInfo])
        {
          # Datei, also Datei zu ZIP-File hinzufgen:
          $e = $zipfile.AddFile($FileObject.FullName, '')
        }
      } catch {
        Write-Warning "ConvertTo-ZIP: $($_.Exception.InnerException.Message)"
      }
    }
  }

  End
  {
    # wird ein selbstextrahierendes Archiv gewnscht?
    if ($SelfExtract)
    {
      # ja, Dateierweiterung ggfs. anpassen:
      $ZipPath = [System.IO.Path]::ChangeExtension($ZipPath, '.exe')

      # Alternative: "WinFormsApplication" statt "ConsoleApplication":
      $zipfile.SaveSelfExtractor($ZipPath, 'ConsoleApplication')
    }
    else
    {
      $zipfile.Save($ZipPath)
    }

    # Objekt freigeben:
    $zipfile.Dispose()
    Write-Warning "Saved ZIP-file to '$ZIPPath'"
  }
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erlutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollstndig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag bernehmen eine Haftung fr Folgen, die durch die Ausfhrung
# dieses Skripts entstehen.
#
################################################################
