﻿Function Test-DynamicParameter
{
  [CmdletBinding()]

  param
  (
    [Parameter(Mandatory=$true)]
    [ValidateSet('FirmaA','FirmaB','FirmaC','FirmaD')]
    # Name der Firma, erlaubt sind nur die im ValidateSet angegebenen
    # Firmennamen 'FirmaA','FirmaB','FirmaC' und 'FirmaD'
    $Firma
  )

  Dynamicparam
  {
    # der zweite (dynamische) Parameter -Abteilung wird nur eingeblendet,
    # wenn bereits mit -Firma eine Firma ausgewählt wurde

    # hier stehen für jede Firma die gültigen Werte, die für
    # -Abteilung angegeben werden dürfen:
    $data = @{
      FirmaA = 'Geschäftsführung', 'Marketing', 'Vertrieb'
      FirmaB = 'Marketing', 'Vertrieb'
      FirmaC = 'Geschäftsführung', 'Außendienst', 'Fuhrpark'
      FirmaD = 'Geschäftsführung', 'Gebäudemanagement', 'Fuhrpark'
    }

    # wurde bereits die Firma mit -Firma angegeben?
    if ($Firma)
    {
      # ja, also dynamischen Parameter anlegen:
      $paramDictionary = New-Object -TypeName System.Management.Automation.RuntimeDefinedParameterDictionary
      $attributeCollection = New-Object -TypeName System.Collections.ObjectModel.Collection[System.Attribute]

      # Parameter-Attribute festsetzen:
      $attribute = New-Object System.Management.Automation.ParameterAttribute
      $attribute.ParameterSetName = '__AllParameterSets'
      $attribute.Mandatory = $false
      $attributeCollection.Add($attribute)

      # gültige Werte festsetzen;
      # die gültigen Abteilungsnamen für eine Firma stehen im Hashtable $data
      # $data.$firma liefert also die Liste der für die jeweilige Firma gültigen
      # Abteilungsnamen.
      # Diese werden als ValidateSet dynamisch dem Parameter hinzugefügt:
      $attribute = New-Object System.Management.Automation.ValidateSetAttribute($data.$firma)
      $attributeCollection.Add($attribute)

      # dynamischen Parameter -Abteilung anlegen:
      $Name = 'Abteilung'
      $dynParam = New-Object -TypeName System.Management.Automation.RuntimeDefinedParameter($Name, [String], $attributeCollection)
      $paramDictionary.Add($Name, $dynParam)

      # Parameter zurückgeben:
      $paramDictionary
    }
  }

  End
  {
    'Die übergebenen Parameter:'
    $PSBoundParameters
  }
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
