﻿<#
.ForwardHelpTargetName Get-ChildItem
.ForwardHelpCategory Cmdlet
#>
Function Get-ChildItem_Ex
{
  [CmdletBinding(DefaultParameterSetName='Items', SupportsTransactions=$true)]
  
  param
  (
    [Parameter(ParameterSetName='Items', Position=0)]
    [string[]]
    $Path,

    [Parameter(ParameterSetName='LiteralItems', Mandatory=$true)]
    [Alias('PSPath')]
    [string[]]
    $LiteralPath,

    [Parameter(Position=1)]
    [string]
    $Filter,

    [string[]]
    $Include,

    [string[]]
    $Exclude,

    [Alias('s')]
    [switch]
    $Recurse,

    [switch]
    $Force,

    [switch]
    $Name,

    [Switch]
    $FileOnly,

    [Switch]
    $FolderOnly,

    [DateTime]
    $Before,

    [DateTime]
    $After,

    [Int64]
    $MaxFileSize,

    [Int64]
    $MinFileSize,

    [Int]
    $OlderThan,

    [Int]
    $NewerThan,

    [String]
    $Sort
  )

  # Initialize pre- and post-Pipeline command store:
  [String[]]$PostPipeline = ''
  [String[]]$Pipeline = '& Get-ChildItem @PSBoundParameters'

  if ($PSBoundParameters.ContainsKey('FileOnly'))
  {
    $PostPipeline += { Where-Object { $_.PSIsContainer -eq $false } }
  }

  if ($PSBoundParameters.ContainsKey('FolderOnly'))
  {
    $PostPipeline += { Where-Object { $_.PSIsContainer -eq $true } }
  }

  if ($PSBoundParameters.ContainsKey('Before'))
  {
    $PostPipeline += { Where-Object { $_.LastWriteTime -lt $Before } }
  }

  if ($PSBoundParameters.ContainsKey('After'))
  {
    $PostPipeline += { Where-Object { $_.LastWriteTime -gt $After } }
  }

  if ($PSBoundParameters.ContainsKey('MaxFileSize'))
  {
    $PostPipeline += { Where-Object { $_.LastWriteTime -le $MaxFileSize -and -not $_.PSIsContainer }}
  }

  if ($PSBoundParameters.ContainsKey('MinFileSize'))
  {
    $PostPipeline += { Where-Object { $_.Length -ge $MinFileSize -and -not $_.PSIsContainer}}
  }

  if ($PSBoundParameters.ContainsKey('OlderThan'))
  {
    $PostPipeline += {Where-Object {((Get-Date) - (New-TimeSpan -Days $OlderThan)) -gt $_.LastWriteTime}}
  }

  if ($PSBoundParameters.ContainsKey('NewerThan'))
  {
    $PostPipeline += {Where-Object {((Get-Date) - (New-TimeSpan -Days $NewerThan)) -lt $_.LastWriteTime}}
  }

  if ($PSBoundParameters.ContainsKey('Sort'))
  {
    $PostPipeline += { Sort-Object -Property $Sort }
  }

  # Remove additional parameters before forwarding them to the original cmdlet:
  $null = $PSBoundParameters.Remove('FileOnly')
  $null = $PSBoundParameters.Remove('FolderOnly')
  $null = $PSBoundParameters.Remove('Before')
  $null = $PSBoundParameters.Remove('After')
  $null = $PSBoundParameters.Remove('MaxFileSize')
  $null = $PSBoundParameters.Remove('MinFileSize')
  $null = $PSBoundParameters.Remove('OlderThan')
  $null = $PSBoundParameters.Remove('NewerThan')
  $null = $PSBoundParameters.Remove('Sort')
  & ([ScriptBlock]::Create((($Pipeline + $PostPipeline) | Where-Object {$_}) -join ' | '))
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
