﻿function Out-StringEx
{
    param
    (
        [Parameter(ValueFromPipeline=$true)]
        $InputObject
    )

    begin
    {
        $wrappedCmd = $ExecutionContext.InvokeCommand.GetCommand('Out-String', 'Cmdlet')
        $scriptCmd = {&  $wrappedCmd -OutVariable script:TranscriptContent -Stream | Out-Null }
        $steppablePipeline = $scriptCmd.GetSteppablePipeline($myInvocation.CommandOrigin)
        $steppablePipeline.Begin($PSCmdlet)
    }


    process
    {
        $steppablePipeline.Process($_)
        $_
    }

    end
    {
        $steppablePipeline.End()
    }   
}

Function Out-Default
{
  param
  (
    [Parameter(ValueFromPipeline=$true)]
    [psobject]
    $InputObject
  )

  Begin
  {
    try {
      $OriginalCommand = $null
      $script:TranscriptContent = $null 
      $wrappedCmd = $ExecutionContext.InvokeCommand.GetCommand('Out-Default', 'Cmdlet')

      if ($global:isTranscribing)
      {
        $scriptCmd = { Out-StringEx | & $wrappedCmd }
      }
      else
      {
        $scriptCmd = {& $wrappedCmd }
      }

      $steppablePipeline = $scriptCmd.GetSteppablePipeline($myInvocation.CommandOrigin)
      $steppablePipeline.Begin($PSCmdlet)
    } catch {
      throw
    }
  }

  Process
  {
    try {
      $steppablePipeline.Process($_)

      if ($global:isTranscribing)
      {
        if ($OriginalCommand -eq $null)
        {  
           $cmd = (Get-Variable -Name MyInvocation -Scope 1).Value.MyCommand.Definition
           $OriginalCommand = '{0}{1}' -f (prompt), $cmd
        }
      }
    } catch {
      throw
    }
  }

  End
  {
    try {
      $steppablePipeline.End()
    } catch {
      throw
    }
    finally
    {
      if ($global:isTranscribing)
      {
        $OriginalCommand | Out-File -Append -FilePath $global:TranscriptPath
        $script:TranscriptContent | Out-File -Append -FilePath $global:TranscriptPath
        $script:TranscriptContent=$null
      }
    }
  }
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
