﻿Function Start-Transcript
{
  param
  (
    [Parameter(Position=0)]
    [ValidateNotNullOrEmpty()]
    [Alias('LiteralPath','PSPath')]
    [string]
    $Path='',

    [switch]
    $Append,

    [switch]
    $Force,

    [Alias('NoOverwrite')]
    [switch]
    $NoClobber
  )

  if($global:isTranscribing)
  {
  Throw 'Start-Transcript : Transcription has already been started. Use the stop-transcript command to stop transcription.'
  }

  $timestamp = Get-Date -Format yyyyMMddHHmmss
  $username = "$env:USERDOMAIN\$env:username"
  $os = Get-WmiObject -Class Win32_OperatingSystem
  $operatingsystem = '{0} ({1} {2} {3})' -f $os.CSName,$os.Caption, $os.Version, $os.CSDVersion

  if ($Path -eq '')
  {
    $Folder = Split-Path (Split-Path $profile) 
    $File = "PowerShell_transcript.$timestamp.txt"
    $Path = Join-Path -Path $Folder -ChildPath $File
  }
  if ($NoClobber)
  {
    if (Test-Path -Path $Path)
    {
      Throw "Start-Transcript: File $Path already exists and NoClobber was specified."
    }
  }


  $header = "
**********************
Windows PowerShell transcript start
Start time: $timestamp
Username  : $username
Machine	  : $operatingsystem
**********************
Transcript started, output file is $path
"
  $header | Out-File -FilePath $Path -Append:$Append -Force:$Force
  Remove-Variable TranscriptContent -Scope Script
  "Transcript started, output file is $path" 
  $global:isTranscribing=$true
  $global:TranscriptPath=$Path
  
}

Function Stop-Transcript
{
  param()

  if ($global:isTranscribing)
  {
    $global:isTranscribing=$false
    "Transcript stopped, output file is $global:TranscriptPath"
  }
  else
  {
    Throw 'Stop-Transcript : An error occurred stopping transcription: The host is not currently transcribing.'
  }
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
