﻿

# eingebaute Funktion, um dieses Modul
# unter einem neuen Namen zu verwenden:
Function New-CustomModule
{
  param
  (
    [Parameter(Mandatory=$true)]
    $Name,

    $Description,

    $Copyright,

    $Author
  )

  # Pfadname für neues Modul
  $Path = Join-Path -Path (Split-Path -Path $profile) -ChildPath "Modules\$Name"

  # existiert dieser Pfad schon?

  if (Test-Path -Path $Path)
  {
    Write-Warning "Ein Modul namens '$Name' existiert bereits."
    break
  }

  # Ordner anlegen
  $null = New-Item -Path $Path -ItemType Directory -Force

  # eigenes Modul dorthin kopieren
  $ModuleName = Split-Path -Path $PSScriptRoot -Leaf
  Copy-Item -Path $PSScriptRoot\ModuleLoader.psm1 -Destination $Path

  # Manifestdatei anpassen
  $hash = Invoke-Expression( (Get-Content "$PSScriptRoot\$ModuleName.psd1" | Out-String) )
  $hash.Remove('FunctionsToExport')
  $hash.Remove('CmdletsToExport')
  $hash.Remove('VariablesToExport')
  $hash.Remove('AliasesToExport')
  $hash.GUID = [system.guid]::NewGuid().ToString()
  $hash.Author = $Author
  $hash.CompanyName = $Company
  $hash.Copyright = $Copyright
  $hash.Description = $Description
  New-ModuleManifest -Path "$Path\$Name.psd1" @hash

  # Manifestdatei zu PS2.0 kompatibel machen
  (Get-Content -Path "$Path\$Name.psd1" -Raw).Replace('RootModule','ModuleToProcess') |
    Set-Content -Path "$Path\$Name.psd1"

  # Modulordner im Explorer öffnen
  Invoke-Item -Path $Path
}

# alle Skripts im Rootordner des Moduls finden:
Get-ChildItem -Path $PSScriptRoot -Filter *.ps1 |
Where-Object { $_.Extension -eq '.ps1' } |
# und alle diese Skripts dot-sourced laden:
ForEach-Object {
  $start = Get-Date
  . $_.FullName
  $ende = Get-Date
  # hat der Ladevorgang verdächtig lange gedauert?
  $dauer = ($ende - $start).TotalSeconds

  Write-Verbose ('Ladezeit des Scripts "{0}": {1} Sekunden.' -f $_.Name, $dauer)

  if ($dauer -gt 2)
  {
    Write-Warning ('Sehr lange Ladezeit des Scripts "{0}": {1} Sekunden.' -f $_.Name, $dauer)
    Write-Warning 'Dies ist ein Hinweis, dass im Skript unbeabsichtigte Befehle ausgeführt werden.'
  }
}

# alle Format-Daten im Rootordner des Moduls finden:
Get-ChildItem -Path $PSScriptRoot -Filter *format.ps1xml |
Where-Object { $_.Extension -eq '.ps1xml' } |
# und diese laden:
Foreach-Object { Update-FormatData -PrependPath $_.FullName }
