﻿Function Export-Certificate
{
  param
  (
    [Parameter(Mandatory=$true,HelpMessage='pfx file name')]
    $Path = "$home\Certificate.pfx",

    [Parameter(Mandatory=$true,HelpMessage='name of installed certificate')]
    $CertificateName,

    [System.Security.SecureString]
    $Password = $null,

    [ValidateSet('cer', 'pfx')]
    $ExportType
  )

  # installiertes Zertifikat auswählen:
  $cert = @(Get-ChildItem cert:\ -Recurse -CodeSigningCert |
    Where-Object { $_.Subject -eq "CN=$CertificateName" })[0]

  # falls ein Zertifikat mit dem angegebenen Namen gefunden wurde,
  # dieses als Datei exportieren:

  if ($cert)
  {
    # wie lautet die Dateierweiterung des angegebenen Dateipfads?
    $ext = [System.IO.Path]::GetExtension($Path).TrimStart('.')

    # falls kein besonderer Exporttyp angegeben wurde, automatisch
    # aus Dateierweiterung festlegen:

    if ($ExportType -eq $null)
    {
      # wurde eine gültige Dateierweiterung angegeben?

      if ('cer', 'pfx' -contains $ext)
      {
        # ja, also diesen Exporttyp wählen:
        $ExportType = $ext
      }
      elseif ($cert.hasPrivateKey)
      {
        # ja, also pfx-Export:
        $ExportType = 'pfx'
        $Path += '.pfx'
      }
      else
      {
        # nein, also cer-Export:
        $ExportType = 'cer'
        $Path += '.cer'
      }
    }

    # tatsächliche Export-Dateierweiterung festlegen:
    $Path = [System.IO.Path]::ChangeExtension($Path, $ExportType)

    # Exportvorgang versuchen:
    try
    {
      # falls pfx-Export und kein Kennwort festgelegt, dieses erfragen:

      if ($ExportType -eq 'pfx' -and $Password -eq $null)
      {
        $Password = Read-Host -AsSecureString -Prompt 'Enter password to protect pfx-file'
      }
      else
      {
        $Password = $null
      }

      # Zertifikat in Datei exportieren:
      $bytes = $cert.Export($ExportType, $Password)
      $filestream = New-Object System.IO.FileStream($Path, 'Create')
      $filestream.Write($bytes, 0, $bytes.Length)
      $filestream.Close()

      # Datei zurückliefern:
      Get-Item -Path $path
    }
    catch
    {
      # falls Export fehlschlägt: darf Zertifikat überhaupt exportiert werden?
      Write-Warning "Certificate '$CertificateName' cannot be exported. It may not be marked as exportable."
    }
  }
  else
  {
    Write-Warning "No certificate found with name '$CertificateName'."
  }
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
