﻿Function Show-RemotingCmdlet
{
  $CredSupport = @{
    Name = 'supports -Credential'
    Expression = {
      if ($_.Parameters.ContainsKey('Credential'))
      {
        'yes'
      }
    }
  }

  # alle Module von Microsoft importieren:
  Write-Progress -Activity 'Finding modules' -Status 'this may take a moment...'
  Get-Module -ListAvailable |
      Where-Object { $_.Author -eq 'Microsoft Corporation' } |
      Import-Module

  $cmdlets = Get-Command -ListImported -CommandType Cmdlet |
      Where-Object RemotingCapability -Like '*Command'
      Write-Progress -Activity 'Finding modules' -Status 'this may take a moment...' -Completed

  $counter = 0

  # alle gefundenen Cmdlets untersuchen:
  $cmdlets |
    Select-Object -Property Name, Synopsis, $CredSupport |
    ForEach-Object {
      $counter++
      Write-Progress -Activity 'Examining Cmdlet' -Status $_.Name -PercentComplete ($counter*100/$cmdlets.Count)

      # Hilfe für Cmdlet abrufen:
      $help = Get-Help -Name $_.Name

      # Hilfe vorhanden?

      if ($help.Remarks -notlike '*go.microsoft.com*')
      {
        # Beschreibung übernehmen:
        $_.Synopsis = $help.Synopsis
      }

      $_
    } |
    Sort-Object -Property Name |
    # in PowerShell 2.0 wird -PassThru nicht unterstützt:
    Out-GridView -Title 'Select Remoting Cmdlet' -PassThru |
    ForEach-Object {
      # Hilfe für die ausgewählten Cmdlets in Fenster öffnen
      # (erfordert PowerShell 3.0):
      Get-Help -Name $_.Name -ShowWindow
    }
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
