﻿Function Set-TokenHighlight
{
  [CmdletBinding(DefaultParameterSetName='Set')]
  param
  (
    # Liste der Tokentypen, die hervorgehoben werden sollen
    # wenn kein Tokentyp angegeben wird, werden die Standardfarben
    # wiederhergestellt
    [Parameter(ParameterSetName='Token', Position=0)]
    [System.Management.Automation.PSTokenType[]]
    $TokenType,

    [Parameter(ParameterSetName='Set', Position=0)]
    [ValidateSet('Off','Grouping', 'Command', 'Keyword')]
    $Group,

    # Transparenzgrad der übrigen Token
    # Vorgabe ist 70 (stark abgeblendet)
    # erlaubt ist ein Wert zwischen 10 und 90 (Prozent)
    [ValidateRange(10,90)]
    $Transparency=70
  )

  # Prozentgrad der Durchsichtigkeit umrechnen in die
  # Byte-Skala der Nicht-Durchsichtigkeit, die benötigt wird:
  [Int]$Level = (100-$Transparency) * 2.55

  if ($PSCmdlet.ParameterSetName -eq 'Set')
  {
    Switch($Group)
    {
      'Command'   { $TokenType = 'Command', 'GroupStart', 'GroupEnd', 'CommandArgument', 'CommandParameter' }
      'Grouping'  { $TokenType = 'GroupStart', 'GroupEnd' }
      'Keyword'   { $TokenType = 'Keyword', 'GroupStart', 'GroupEnd', 'Attribute', 'CommandArgument' }
      'Off'       {}

    }
  }

  # alle Standardfarben wiederherstellen:
  $psise.Options.RestoreDefaultTokenColors()

  # wenn überhaupt in $TokenType etwas angegeben wurde...
  if ($TokenType)
  {
    # dann die Namen der Token lesen...
    ([String[]]$psise.Options.TokenColors.Keys) |
    # nur diejenigen bearbeiten, die NICHT angegeben wurden...
    Where-Object { $TokenType -notcontains $_ } |
    ForEach-Object {
      # und deren Transparenz neu setzen:
      $color = $psISE.Options.TokenColors[$_]
      $color.A = $Level
      $psISE.Options.TokenColors[$_] = $color
    }
  }
}


################################################################
#
# VERWENDUNG DIESES SKRIPTS AUF EIGENE GEFAHR / OHNE ZUSICHERUNG
#
# Wichtiger Hinweis:
#
# Dieses PowerShell-Skript stammt aus den Begleitmedien des Buches
# "Scripting mit Windows PowerShell 3.0 - Der Workshop" von 
# Dr. Tobias Weltner, erschienen 2013 beim Verlag Microsoft Press
# (ISBN-13: 978-3-86645-687-7).
#
# Das Skript und seine Funktion werden im Buch erläutert und liegen
# in dieser Datei vor, um die Eingabe des Codes zu erleichtern.
#
# Bitte vergewissern Sie sich im Kontext des Buches und des jeweiligen
# Kapitels, was die Aufgabe und Funktionsweise dieses Skripts ist, 
# und dass das Skript vollständig dem abgedruckten Code im Buch entspricht. 
#
# Weder Autor noch Verlag übernehmen eine Haftung für Folgen, die durch die Ausführung
# dieses Skripts entstehen.
#
################################################################
